@@ -73,15 +73,17 @@
 ring_alloc_logic_attr::ring_alloc_logic_attr()
     : m_ring_alloc_logic(RING_LOGIC_PER_INTERFACE)
     , m_user_id_key(0)
+    , m_use_locks(true)
 {
     m_mem_desc.iov_base = NULL;
     m_mem_desc.iov_len = 0;
     init();
 }
 
-ring_alloc_logic_attr::ring_alloc_logic_attr(ring_logic_t ring_logic)
+ring_alloc_logic_attr::ring_alloc_logic_attr(ring_logic_t ring_logic, bool use_locks)
     : m_ring_alloc_logic(ring_logic)
     , m_user_id_key(0)
+    , m_use_locks(use_locks)
 {
     m_mem_desc.iov_base = NULL;
     m_mem_desc.iov_len = 0;
@@ -93,6 +95,7 @@ ring_alloc_logic_attr::ring_alloc_logic_attr(const ring_alloc_logic_attr &other)
     , m_ring_alloc_logic(other.m_ring_alloc_logic)
     , m_user_id_key(other.m_user_id_key)
     , m_mem_desc(other.m_mem_desc)
+    , m_use_locks(other.m_use_locks)
 {
 }
 
@@ -117,6 +120,7 @@ void ring_alloc_logic_attr::init()
     HASH_ITER(m_user_id_key, uint64_t);
     HASH_ITER(m_mem_desc.iov_base, uintptr_t);
     HASH_ITER(m_mem_desc.iov_len, size_t);
+    HASH_ITER(m_use_locks, bool);
 
     m_hash = h;
 #undef HASH_ITER
@@ -146,12 +150,21 @@ void ring_alloc_logic_attr::set_user_id_key(uint64_t user_id_key)
     }
 }
 
+void ring_alloc_logic_attr::set_use_locks(bool use_locks)
+{
+    if (m_use_locks != use_locks) {
+        m_use_locks = use_locks;
+        init();
+    }
+}
+
 const std::string ring_alloc_logic_attr::to_str() const
 {
     std::stringstream ss;
 
     ss << "allocation logic " << m_ring_alloc_logic << " key " << m_user_id_key << " user address "
-       << m_mem_desc.iov_base << " user length " << m_mem_desc.iov_len;
+       << m_mem_desc.iov_base << " user length " << m_mem_desc.iov_len << " use locks "
+       << !!m_use_locks;
 
     return ss.str();
 }
@@ -1353,7 +1366,7 @@ void net_device_val_eth::configure()
     }
 }
 
-int net_device_val::get_priority_by_tc_class(uint32_t tc_class)
+uint32_t net_device_val::get_priority_by_tc_class(uint32_t tc_class)
 {
     tc_class_priority_map::iterator it = m_class_prio_map.find(tc_class);
     if (it == m_class_prio_map.end()) {
@@ -1411,12 +1424,11 @@ ring *net_device_val_eth::create_ring(resource_allocation_key *key)
 {
     ring *ring = NULL;
 
-    NOT_IN_USE(key);
-
     try {
         switch (m_bond) {
         case NO_BOND:
-            ring = new ring_eth(get_if_idx());
+            ring = new ring_eth(get_if_idx(), nullptr, RING_ETH, true,
+                                (key ? key->get_use_locks() : true));
             break;
         case ACTIVE_BACKUP:
         case LAG_8023ad:
@@ -1617,15 +1629,6 @@ bool net_device_val::verify_qp_creation(const char *ifname, enum ibv_qp_type qp_
     }
     VALGRIND_MAKE_MEM_DEFINED(channel, sizeof(ibv_comp_channel));
     context = p_ib_ctx->get_ibv_context();
-#if defined(DEFINED_NGINX)
-    /*
-     * For NGINX scenario we may want to distribute CQs across multiple
-     * CPUs to improve CPS in case of multiple NGINX worker processes.
-     */
-    if (safe_mce_sys().nginx_distribute_cq_interrupts) {
-        comp_vector = g_worker_index % context->num_comp_vectors;
-    }
-#endif
     cq = xlio_ibv_create_cq(context, safe_mce_sys().tx_num_wr, (void *)this, channel, comp_vector,
                             &attr);
     if (!cq) {
