@@ -98,8 +98,8 @@ qp_mgr *ring_eth::create_qp_mgr(struct qp_mgr_desc *desc)
     return new qp_mgr_eth_mlx5(desc, get_tx_num_wr(), m_partition);
 }
 
-ring_simple::ring_simple(int if_index, ring *parent, ring_type_t type)
-    : ring_slave(if_index, parent, type)
+ring_simple::ring_simple(int if_index, ring *parent, ring_type_t type, bool use_locks)
+    : ring_slave(if_index, parent, type, use_locks)
     , m_p_ib_ctx(NULL)
     , m_p_qp_mgr(NULL)
     , m_p_cq_mgr_rx(NULL)
@@ -147,7 +147,6 @@ ring_simple::ring_simple(int if_index, ring *parent, ring_type_t type)
     memset(&m_lro, 0, sizeof(m_lro));
 
     INIT_LIST_HEAD(&m_socketxtreme.ec_list);
-    m_socketxtreme.completion = NULL;
 }
 
 ring_simple::~ring_simple()
@@ -346,11 +345,11 @@ void ring_simple::create_resources()
 #endif /* DEFINED_UTLS */
 
     m_flow_tag_enabled = !safe_mce_sys().disable_flow_tag && m_p_ib_ctx->get_flow_tag_capability();
-#if defined(DEFINED_NGINX)
-    if (safe_mce_sys().actual_nginx_workers_num > 0) {
+#if defined(DEFINED_NGINX) || defined(DEFINED_ENVOY)
+    if (g_p_app->type != APP_NONE && g_p_app->get_worker_id() >= 0) {
         m_flow_tag_enabled = false;
     }
-#endif // DEFINED_NGINX
+#endif
     ring_logdbg("ring attributes: m_flow_tag_enabled = %d", m_flow_tag_enabled);
 
     m_p_rx_comp_event_channel = ibv_create_comp_channel(
@@ -451,60 +450,54 @@ int ring_simple::poll_and_process_element_tx(uint64_t *p_cq_poll_sn)
 int ring_simple::socketxtreme_poll(struct xlio_socketxtreme_completion_t *xlio_completions,
                                    unsigned int ncompletions, int flags)
 {
-    int ret = 0;
     int i = 0;
+    unsigned int pkts = 0U;
+    bool do_poll = true;
 
     if (likely(xlio_completions) && ncompletions) {
-
         if ((flags & SOCKETXTREME_POLL_TX) && list_empty(&m_socketxtreme.ec_list)) {
             uint64_t poll_sn = 0;
             const std::lock_guard<decltype(m_lock_ring_tx)> lock(m_lock_ring_tx);
-            while (m_p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn) > 0) {
-            }
+            m_p_cq_mgr_tx->poll_and_process_element_tx(&poll_sn);
         }
 
         const std::lock_guard<decltype(m_lock_ring_rx)> lock(m_lock_ring_rx);
-        m_socketxtreme.completion = xlio_completions;
         while (!g_b_exit && (i < (int)ncompletions)) {
-            m_socketxtreme.completion->events = 0;
-            /* Check list size to avoid locking */
             if (!list_empty(&m_socketxtreme.ec_list)) {
                 ring_ec *ec = get_ec();
                 if (ec) {
-                    memcpy(m_socketxtreme.completion, &ec->completion, sizeof(ec->completion));
+                    memcpy(xlio_completions, &ec->completion, sizeof(ec->completion));
                     ec->clear();
-                    m_socketxtreme.completion++;
+                    xlio_completions++;
                     i++;
                 }
-            } else {
-                /* Internal thread can raise event on this stage before we
-                 * start rx processing. In this case we can return event
-                 * in right order. It is done to avoid locking and
-                 * may be it is not so critical
-                 */
+            } else if (likely(do_poll)) {
                 mem_buf_desc_t *desc = m_p_cq_mgr_rx->poll_and_process_socketxtreme();
                 if (likely(desc)) {
-                    desc->rx.socketxtreme_polled = true;
                     rx_process_buffer(desc, NULL);
-                    if (m_socketxtreme.completion->events) {
-                        m_socketxtreme.completion++;
-                        i++;
+
+                    // To avoid too many unflushed GRO sessions, We flush GRO and continue polling.
+                    // Otherwise this loop may continue for too long and result in way more
+                    // completions than ncompletions, what is not optimal for performance.
+                    // Not each packet results in a real completion but this check is good enough.
+                    if (++pkts >= ncompletions) {
+                        m_gro_mgr.flush_all(NULL);
+                        pkts = 0U;
                     }
                 } else {
-                    break;
+                    m_gro_mgr.flush_all(NULL);
+                    do_poll = false;
                 }
+            } else {
+                break;
             }
         }
-
-        m_socketxtreme.completion = NULL;
-
-        ret = i;
     } else {
-        ret = -1;
+        i = -1;
         errno = EINVAL;
     }
 
-    return ret;
+    return i;
 }
 
 int ring_simple::wait_for_notification_and_process_element(int cq_channel_fd,
@@ -900,8 +893,6 @@ void ring_simple::init_tx_buffers(uint32_t count)
 {
     request_more_tx_buffers(PBUF_RAM, count, m_tx_lkey);
     m_tx_num_bufs = m_tx_pool.size();
-    request_more_tx_buffers(PBUF_ZEROCOPY, count, 0);
-    m_zc_num_bufs = m_zc_pool.size();
 }
 
 void ring_simple::inc_cq_moderation_stats(size_t sz_data)
@@ -1191,7 +1182,7 @@ uint32_t ring_simple::get_tx_user_lkey(void *addr, size_t length, void *p_mappin
             lkey = iter->second;
         } else {
             lkey = m_p_ib_ctx->user_mem_reg(addr, length, XLIO_IBV_ACCESS_LOCAL_WRITE);
-            if (lkey == (uint32_t)(-1)) {
+            if (lkey == LKEY_ERROR) {
                 ring_logerr("Can't register user memory addr %p len %lx", addr, length);
             } else {
                 m_user_lkey_map[addr] = lkey;
