@@ -56,18 +56,13 @@
 #define fdcoll_logfunc    __log_func
 
 fd_collection *g_p_fd_collection = NULL;
-#if defined(DEFINED_NGINX)
-fd_collection *g_p_fd_collection_parent_process = NULL;
-int g_p_fd_collection_size_parent_process = 0;
-#endif // DEFINED_NGINX
 
 fd_collection::fd_collection()
     : lock_mutex_recursive("fd_collection")
     , m_b_sysvar_offloaded_sockets(safe_mce_sys().offloaded_sockets)
 #if defined(DEFINED_NGINX)
     // Avoid using socket pool for the master process (which doesn't have parent fd_collection)
-    , m_use_socket_pool(safe_mce_sys().nginx_udp_socket_pool_size &&
-                        g_p_fd_collection_parent_process)
+    , m_use_socket_pool(safe_mce_sys().nginx_udp_socket_pool_size && g_p_app->get_worker_id() >= 0)
     , m_socket_pool_size(safe_mce_sys().nginx_udp_socket_pool_size)
     , m_socket_pool_counter(0)
 #endif
@@ -156,7 +151,7 @@ void fd_collection::clear()
         p_sfd_api->clean_obj();
     }
 
-    g_global_stat_static.n_pending_sockets = 0U;
+    g_global_stat_static.n_pending_sockets = 0;
 
     /* Clean up all left overs sockinfo
      */
@@ -522,7 +517,7 @@ int fd_collection::add_cq_channel_fd(int cq_ch_fd, ring *p_ring)
     return 0;
 }
 
-int fd_collection::del_sockfd(int fd, bool b_cleanup /*=false*/)
+int fd_collection::del_sockfd(int fd, bool b_cleanup /*=false*/, bool is_for_udp_pool /*=false*/)
 {
     int ret_val = -1;
     socket_fd_api *p_sfd_api;
@@ -544,8 +539,12 @@ int fd_collection::del_sockfd(int fd, bool b_cleanup /*=false*/)
             // Delete it from fd_col and add it to pending_to_remove list.
             // This socket will be handled and destroyed now by fd_col.
             // This will be done from fd_col timer handler.
+            // Used for UDP socket pool as well
+            // so closed UDP sockets will be deleted at the end of the world
             if (m_p_sockfd_map[fd] == p_sfd_api) {
-                ++g_global_stat_static.n_pending_sockets;
+                if (!is_for_udp_pool) {
+                    ++g_global_stat_static.n_pending_sockets;
+                }
                 m_p_sockfd_map[fd] = NULL;
                 m_pending_to_remove_lst.push_front(p_sfd_api);
             }
@@ -672,7 +671,7 @@ void fd_collection::handle_socket_pool(int fd)
     }
 
     if (m_socket_pool_counter >= m_socket_pool_size) {
-        fdcoll_logdbg("Nginx worker num %d reached max UDP socket pool size (%d).", g_worker_index,
+        fdcoll_logdbg("Worker %d reached max UDP socket pool size (%d).", g_p_app->get_worker_id(),
                       m_socket_pool_size);
         m_use_socket_pool = false;
         return;
